<?php

namespace Drupal\epsilon_harmony\Services;

use function GuzzleHttp\json_decode;
use Exception as Exception;
use GuzzleHttp\Exception\ClientException;
use Drupal\Component\Serialization\Json;
use Drupal\epsilon_harmony\StatusCodes;

/**
 * Description for API Factory of Harmony API.
 *
 * @author Darshan Choudhary
 */
class EpsilonApiFactory extends EpsilonConnectionFactory {

  /**
   * Get a new access token or an existing one for the API.
   *
   * @return string
   *   Returns the access token.
   */
  public static function getToken($object, $test = FALSE) {
    if ($test == TRUE || ($object->getTokenTimeout() == NULL || (time() - $object->getTokenTimeout() > 3600))) {
      try {
        $url = $object->tokenUrl . '/Epsilon/oauth2/access_token';

        // Form data to be passed.
        $data['username']   = $object->getUsername();
        $data['password']   = $object->getPassword();
        $data['scope']      = 'cn mail sn givenname uid employeeNumber';
        $data['grant_type'] = 'password';

        $headers = [
          'Authorization' => 'Basic ' . $object->getBaseToken(),
          // 'Authorization' => 'Basic ddd',
          'Content-Type' => 'application/x-www-form-urlencoded',
        ];

        $response = \Drupal::httpClient()->post($url, [
          'headers' => $headers,
          'form_params' => $data,
        ]);

        $response_data_json = $response->getBody()->getContents();
        $response_data = (array) Utils::jsonDecode($response_data_json);

        $values = [
          'endpoint' => $url,
          'uid' => \Drupal::currentUser()->id(),
          'method' => 'POST',
          'status_code' => $response->getStatusCode(),
          'status_message' => $response->getReasonPhrase(),
          'header' => Json::encode($headers),
          'request' => Json::encode($data),
          'response' => Json::encode($response_data),
        ];
        $object->logEpsilon($values);

        if ($response->getStatusCode() == StatusCodes::HTTP_OK && !empty($response_data)) {
          \Drupal::state()->set('epsilon_harmony_token_timeout', time());
          \Drupal::state()->set('epsilon_harmony_access_token', $response_data['access_token']);

          \Drupal::messenger()->addStatus(t('Token refreshed successfully.'), TRUE);

          return $response_data['access_token'];
        }
        else {
          throw new Exception('Something went wrong while fetching the token. Please check the logs for further details.');
        }
      }
      catch (Exception $e) {
        if ($e->getResponse() != NULL) {
          $response = (string) $e->getResponse()->getBody();
          $values = [
            'endpoint' => $url,
            'uid' => \Drupal::currentUser()->id(),
            'method' => 'POST',
            'status_code' => $e->getCode(),
            'status_message' => (isset(Json::decode($response)['fault']['faultstring'])) ? Json::decode($response)['fault']['faultstring'] : NULL,
            'header' => Json::encode($headers),
            'request' => Json::encode($data),
            'response' => $response,
          ];
          $log = $object->logEpsilon($values);
          \Drupal::messenger()->addError(t('Failure!! Please check the epsilon log @link for further details.', [
            '@link' => $log->toLink()->toString(),
          ]), TRUE);
        }
        else {
          \Drupal::messenger()->addError(t('Something went wrong! Please check drupal logs for further details.'), TRUE);
        }

        return NULL;
      }
    }
    else {
      return self::$accessToken;
    }
  }

  /**
   * Created a record on Epsilon DB by calling the API.
   *
   * @param array $record
   *   Data to be passed through API.
   *
   * @return array
   *   Returns the response recieved from the API.
   */
  public function createRecord(array $record) {
    try {
      $url = $this->apiUrl . "/v4/profiles/records";

      $headers = [
        'Authorization' => 'Bearer ' . self::getToken($this),
        'X-OUID' => $this->getXouid(),
        'Content-Type' => 'application/json',
      ];

      // HTTP Form request.
      $response = \Drupal::httpClient()->post($url, [
        'headers' => $headers,
        'json' => $record,
      ]);

      $response_data_json = $response->getBody()->getContents();
      $response_data = (array) Utils::jsonDecode($response_data_json);

      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'POST',
        'status_code' => $response->getStatusCode(),
        'status_message' => $response->getReasonPhrase(),
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => Json::encode($response_data),
      ];
      $log = self::logEpsilon($values);

      if ($response->getStatusCode() == StatusCodes::HTTP_OK && !empty($response_data)) {
        \Drupal::messenger()->addStatus(t('Record successfully created. Please check the epsilon log @link for further details.', [
          '@link' => $log->toLink()->toString(),
        ]), TRUE);
        return $response_data;
      }
      else {
        throw new Exception('Create record failed. Please check the logs for further details.');
      }
    }
    catch (ClientException $e) {
      $response = (string) $e->getResponse()->getBody();
      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'POST',
        'status_code' => $e->getCode(),
        'status_message' => Json::decode($response)['resultCode'],
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => $response,
      ];
      $log = self::logEpsilon($values);

      \Drupal::messenger()->addError(t('Create record failed. Please check the log @link for further details.', [
        '@link' => $log->toLink()->toString(),
      ]), TRUE);
    }
  }

  /**
   * Updates the record on Epsilon DB by calling the API.
   *
   * @param array $record
   *   Data to be passed through API.
   *
   * @return array
   *   Returns the response recieved from the API.
   */
  public function updateRecord(array $record) {
    try {
      $url = $this->apiUrl . "/v4/profiles/records/" . $record['CustomerKey'];

      $headers = [
        'Authorization' => 'Bearer ' . self::getToken($this),
        'X-OUID' => $this->getXouid(),
        'Content-Type' => 'application/json',
      ];

      // HTTP Form request.
      $response = \Drupal::httpClient()->put($url, [
        'headers' => $headers,
        'json' => $record,
      ]);

      $response_data_json = $response->getBody()->getContents();
      $response_data = (array) Utils::jsonDecode($response_data_json);

      // Log the API.
      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'PUT',
        'status_code' => $response->getStatusCode(),
        'status_message' => $response->getReasonPhrase(),
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => Json::encode($response_data),
      ];
      $log = self::logEpsilon($values);

      if ($response->getStatusCode() == StatusCodes::HTTP_OK && !empty($response_data)) {
        \Drupal::messenger()->addStatus(t('Record successfully updated. Please check the epsilon log @link for further details.', [
          '@link' => $log->toLink()->toString(),
        ]), TRUE);
        return $response_data;
      }
      else {
        throw new Exception('Update record failed. Please check the epsilon logs for further details.');
      }
    }
    catch (ClientException $e) {
      $response = (string) $e->getResponse()->getBody();
      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'PUT',
        'status_code' => $e->getCode(),
        'status_message' => Json::decode($response)['resultCode'],
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => $response,
      ];
      $log = self::logEpsilon($values);

      \Drupal::messenger()->addStatus(t('Record update failed. Please check the epsilon log @link for further details.', [
        '@link' => $log->toLink()->toString(),
      ]), TRUE);
    }
  }

  /**
   * Deletes the record on Epsilon DB by calling the API.
   *
   * @param string $customer_key
   *   Data to be passed through API.
   *
   * @return array
   *   Returns the response recieved from the API.
   */
  public function deleteRecord($customer_key) {
    try {
      $url = $this->apiUrl . "/v4/profiles/records/" . $customer_key;

      $headers = [
        'Authorization' => 'Bearer ' . self::getToken($this),
        'X-OUID' => $this->getXouid(),
        'Content-Type' => 'application/json',
      ];

      // HTTP Form request.
      $response = \Drupal::httpClient()->delete($url, [
        'headers' => $headers,
      ]);

      $response_data_json = $response->getBody()->getContents();
      $response_data = (array) Utils::jsonDecode($response_data_json);

      // Log the API.
      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'DELETE',
        'status_code' => $response->getStatusCode(),
        'status_message' => $response->getReasonPhrase(),
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => Json::encode($response_data),
      ];
      $log = self::logEpsilon($values);

      if ($response->getStatusCode() == StatusCodes::HTTP_OK && !empty($response_data)) {
        \Drupal::messenger()->addStatus(t('Record successfully deleted. Please check the epsilon log @link for further details.', [
          '@link' => $log->toLink()->toString(),
        ]), TRUE);
        return $response_data;
      }
      else {
        throw new Exception('Record deletion failed. Please check the epsilon logs for further details.');
      }
    }
    catch (ClientException $e) {
      $response = (string) $e->getResponse()->getBody();
      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'DELETE',
        'status_code' => $e->getCode(),
        'status_message' => Json::decode($response)['resultCode'],
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => $response,
      ];
      $log = self::logEpsilon($values);

      \Drupal::messenger()->addStatus(t('Record deletion failed. Please check the epsilon log @link for further details.', [
        '@link' => $log->toLink()->toString(),
      ]), TRUE);
    }
  }

  /**
   * Send a real time email via Epsilon.
   *
   * @param string $message_id
   *   Message ID key to be passed for a particular template to trigger.
   * @param array $record
   *   Data to be passed through API.
   *
   * @return array
   *   Returns the response recieved from the API.
   */
  public function sendMessage($message_id = NULL, $record = []) {
    try {
      if ($message_id == NULL) {
        throw new Exception('Message ID not available.');
      }

      $url = $this->apiUrl . "/v3/messages/" . $this->getmessageId($message_id) . "/send";

      $headers = [
        'Authorization' => 'Bearer ' . self::getToken($this),
        'X-OUID' => $this->getXouid(),
        'Content-Type' => 'application/json',
      ];

      // HTTP Form request.
      $response = \Drupal::httpClient()->put($url, [
        'headers' => $headers,
        'json' => $record,
      ]);

      $response_data_json = $response->getBody()->getContents();
      $response_data = (array) Utils::jsonDecode($response_data_json);

      // Log the API.
      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'PUT',
        'status_code' => $response->getStatusCode(),
        'status_message' => $response->getReasonPhrase(),
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => Json::encode($response_data),
      ];
      $log = self::logEpsilon($values);

      if ($response->getStatusCode() == StatusCodes::HTTP_OK && !empty($response_data)) {
        \Drupal::messenger()->addStatus(t('Message successfully sent. Please check the epsilon logs @link for further details.', [
          '@link' => $log->toLink()->toString(),
        ]), TRUE);
        return $response_data;
      }
    }
    catch (ClientException $e) {
      $response = (string) $e->getResponse()->getBody();
      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'PUT',
        'status_code' => $e->getCode(),
        'status_message' => Json::decode($response)['resultCode'],
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => $response,
      ];
      $log = self::logEpsilon($values);

      \Drupal::messenger()->addStatus(t('Message sending failed. Please check the epsilon log @link for further details.', [
        '@link' => $log->toLink()->toString(),
      ]), TRUE);

    }
  }

  /**
   * Retrieves the record on Epsilon DB by calling the API.
   *
   * @param string $customer_key
   *   Data to be passed through API.
   *
   * @return array
   *   Returns the response recieved from the API.
   */
  public function retrieveRecord($customer_key) {
    try {
      $url = $this->apiUrl . "/v4/profiles/records/" . $customer_key;

      $headers = [
        'Authorization' => 'Bearer ' . self::getToken($this),
        'X-OUID' => $this->getXouid(),
        'Content-Type' => 'application/json',
      ];

      // HTTP Form request.
      $response = \Drupal::httpClient()->get($url, [
        'headers' => $headers,
      ]);

      $response_data_json = $response->getBody()->getContents();
      $response_data = (array) Utils::jsonDecode($response_data_json);

      // Log the API.
      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'GET',
        'status_code' => $response->getStatusCode(),
        'status_message' => $response->getReasonPhrase(),
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => Json::encode($response_data),
      ];
      $log = self::logEpsilon($values);

      if ($response->getStatusCode() == StatusCodes::HTTP_OK && !empty($response_data)) {
        \Drupal::messenger()->addStatus(t('Record retrieved successfully. Please check the epsilon log @link for further details.', [
          '@link' => $log->toLink()->toString(),
        ]), TRUE);
        return $response_data;
      }
      else {
        throw new Exception('Record retrieve failed. Please check the epsilon logs for further details.');
      }
    }
    catch (ClientException $e) {
      $response = (string) $e->getResponse()->getBody();
      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'GET',
        'status_code' => $e->getCode(),
        'status_message' => Json::decode($response)['resultCode'],
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => $response,
      ];
      $log = self::logEpsilon($values);

      \Drupal::messenger()->addStatus(t('Record retrieve failed. Please check the epsilon log @link for further details.', [
        '@link' => $log->toLink()->toString(),
      ]), TRUE);
    }
  }

  /**
   * Create a list record on Epsilon DB by calling the API.
   *
   * @param array $record
   *   Data to be passed through API.
   * @param string $list_id
   *   Data to be passed through API.
   *
   * @return array
   *   Returns the response recieved from the API.
   */
  public function createListRecord(array $record, $list_id = NULL) {
    try {
      $url = $this->apiUrl . "/v4/lists/" . $this->getlistId($list_id) . "/records";

      $headers = [
        'Authorization' => 'Bearer ' . self::getToken($this),
        'X-OUID' => $this->getXouid(),
        'Content-Type' => 'application/json',
      ];

      // HTTP Form request.
      $response = \Drupal::httpClient()->post($url, [
        'headers' => $headers,
        'json' => $record,
      ]);

      $response_data_json = $response->getBody()->getContents();
      $response_data = (array) Utils::jsonDecode($response_data_json);

      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'POST',
        'status_code' => $response->getStatusCode(),
        'status_message' => $response->getReasonPhrase(),
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => Json::encode($response_data),
      ];
      $log = self::logEpsilon($values);

      if ($response->getStatusCode() == StatusCodes::HTTP_OK && !empty($response_data)) {
        \Drupal::messenger()->addStatus(t('List Record successfully created. Please check the epsilon log @link for further details.', [
          '@link' => $log->toLink()->toString(),
        ]), TRUE);
        return $response_data;
      }
      else {
        throw new Exception('Create record failed. Please check the logs for further details.');
      }
    }
    catch (ClientException $e) {
      $response = (string) $e->getResponse()->getBody();
      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'POST',
        'status_code' => $e->getCode(),
        'status_message' => Json::decode($response)['resultCode'],
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => $response,
      ];
      $log = self::logEpsilon($values);

      \Drupal::messenger()->addError(t('Create record failed. Please check the log @link for further details.', [
        '@link' => $log->toLink()->toString(),
      ]), TRUE);
    }
  }

  /**
   * Update a list record on Epsilon DB by calling the API.
   *
   * @param array $record
   *   Data to be passed through API.
   * @param string $list_id
   *   Data to be passed through API.
   *
   * @return array
   *   Returns the response recieved from the API.
   */
  public function updateListRecord(array $record, $list_id = NULL) {
    try {
      $url = $this->apiUrl . "/v4/lists/" . $this->getlistId($list_id) . "/records/" . $record['CustomerKey'];

      $headers = [
        'Authorization' => 'Bearer ' . self::getToken($this),
        'X-OUID' => $this->getXouid(),
        'Content-Type' => 'application/json',
      ];

      // HTTP Form request.
      $response = \Drupal::httpClient()->put($url, [
        'headers' => $headers,
        'json' => $record,
      ]);

      $response_data_json = $response->getBody()->getContents();
      $response_data = (array) Utils::jsonDecode($response_data_json);

      // Log the API.
      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'PUT',
        'status_code' => $response->getStatusCode(),
        'status_message' => $response->getReasonPhrase(),
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => Json::encode($response_data),
      ];
      $log = self::logEpsilon($values);

      if ($response->getStatusCode() == StatusCodes::HTTP_OK && !empty($response_data)) {
        \Drupal::messenger()->addStatus(t('List Record successfully updated. Please check the epsilon log @link for further details.', [
          '@link' => $log->toLink()->toString(),
        ]), TRUE);
        return $response_data;
      }
      else {
        throw new Exception('Update record failed. Please check the epsilon logs for further details.');
      }
    }
    catch (ClientException $e) {
      $response = (string) $e->getResponse()->getBody();
      $values = [
        'endpoint' => $url,
        'uid' => \Drupal::currentUser()->id(),
        'method' => 'PUT',
        'status_code' => $e->getCode(),
        'status_message' => Json::decode($response)['resultCode'],
        'header' => Json::encode($headers),
        'request' => Json::encode($record),
        'response' => $response,
      ];
      $log = self::logEpsilon($values);

      \Drupal::messenger()->addStatus(t('Record update failed. Please check the epsilon log @link for further details.', [
        '@link' => $log->toLink()->toString(),
      ]), TRUE);
    }
  }

  /**
   * Checks if the configurations are valid by making a token call.
   *
   * @return string
   *   Return the access key for testing.
   */
  public function testApi() {
    return self::getToken($this, TRUE);
  }

  /**
   * Logs the epsilon API call in DB.
   *
   * @param array $record
   *   Array of the record to be logged.
   *
   * @return object
   *   Return the log created.
   */
  protected function logEpsilon($record = []) {
    $log = \Drupal::service('entity_type.manager')->getStorage('epsilon_harmony_log')->create($record);
    $log->save();
    return $log;
  }

}
